---------------------------------------------------------------------------
-- This VHDL file was developed by Daniel Llamocca (2017).  It may be
-- freely copied and/or distributed at no cost.  Any persons using this
-- file for any purpose do so at their own risk, and are responsible for
-- the results of such use.  Daniel Llamocca does not guarantee that
-- this file is complete, correct, or fit for any particular purpose.
-- NO WARRANTY OF ANY KIND IS EXPRESSED OR IMPLIED.  This notice must
-- accompany any copy of this file.
--------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- It generates a group (NC cores) of NI-to-NO LUTs.
entity LUT_group is
	generic ( NC: INTEGER:= 3; -- number of cores (NI-to-NO LUTs)
			  NI: INTEGER:= 8; -- number of input bits of each core
			  NO: INTEGER:= 8; -- number of output bits of each core
			  file_LUT: STRING:= "LUT_values.txt";  -- contains all the functions allowed (one after the other)
			  SAME: STRING:= "YES"; -- "YES": Same function F is applied to all 'NC' cores.
			                        -- "NO". Each of the 'NC' cores will use a different function (function 'i' for core 'i'), i = 1,...,NC
			                        --       Here, 'file_LUT' should contain NC sets of function values.
			  F: INTEGER:= 1); -- type of function (1..5) --> as many as permitedby file_LUT. Valid only if SAME = "YES"
	port ( dyn_in: in std_logic_vector (NC*NI - 1 downto 0);
		   dyn_out: out std_logic_vector (NC*NO - 1 downto 0));
           -- Data Order: |core 1|core 2|...|core NC|
           --    core 1:  dyn_in(NC*NI -1 downto (NC-1)*NI), dyn_out(NC*NO -1 downto (NC-1)*NO) . We assign function 1 (if SAME = "NO")
           --      ...
           --    core NC: dyn_in(NI -1    downto 0),         dyn_out(NI-1 downto 0). We assign function NC (if SAME = "NO")

end LUT_group;

architecture structure of LUT_group is

	component LUT_NItoNO
		generic ( NI: INTEGER:= 8;
					 NO: INTEGER:= 8;
					 F: INTEGER:= 5; -- type of function (1..5)
					 file_LUT: STRING:= "LUT_values.txt"); -- file where the numerical values are stored
		-- Function F = 1: Gamma correction - 0.5
		-- Function F = 2: Gamma correction - 2
		-- Function F = 3: Inverse Gamma correction - 0.5
		-- Function F = 4: Inverse Gamma correction - 2
		-- Function F = 5: Contrast Stretching - Alpha = 2, m = 0.5
		-- ...
		port (	LUT_in: in std_logic_vector (NI-1 downto 0);
				LUT_out: out std_logic_vector (NO-1 downto 0)
				);
	end component;
	
	type chunkI is array (NC - 1 downto 0) of std_logic_vector (NI-1 downto 0);
	type chunkO is array (NC - 1 downto 0) of std_logic_vector (NO-1 downto 0);
	
	signal in_data: chunkI;
	signal out_data: chunkO;
	
begin

-- dyn_in, dyn_out: |core 1|core 2|core 3|...|core NC|
   -- dyn_in(core 1) = in_data(0)
   -- dyn_in(core 2) = in_data(1)
   -- dyn_in(core NC) = in_data(NC-1)
gs: for i in 0 to NC - 1 generate
			in_data(NC-i-1) <= dyn_in( (i+1)*NI - 1 downto i*NI );
			dyn_out( (i+1)*NO - 1 downto i*NO ) <= out_data(NC-i-1);
	 end generate;

ns: if SAME = "YES" generate
        ga: for i in 0 to NC - 1 generate
                gi: LUT_NItoNO generic map (NI => NI, NO => NO, F => F, file_LUT => file_LUT)
                    port map (LUT_in => in_data(i), LUT_out => out_data(i));			 
             end generate;
    end generate;

nq: if SAME = "NO" generate    
        ga: for i in 0 to NC - 1 generate
                gi: LUT_NItoNO generic map (NI => NI, NO => NO, F => i+1, file_LUT => file_LUT)
                    port map (LUT_in => in_data(i), LUT_out => out_data(i));             
            end generate;
     end generate;
end structure;